#!/usr/bin/perl -w
# Fasta 形式の cDNA 配列を MacClade Nexus 形式に変換する
# perl FasToNex_AA.pl .fas$

use strict;
use warnings;

my $keyWord = $ARGV[0];

# カレントディレクトリのファイルをリストにする．
opendir(DIR,".");
my @curDir = readdir(DIR);
close (DIR);

foreach my $infile (@curDir) {
chomp($infile);
  if ($infile =~ /$keyWord/) {             # ファイル一覧からlキーワードを含むファイルだけを処理する．
    &FasToNex_cDNA($infile);
  } 
}

exit;


######### サブルーチン

#  Fasta 形式をハッシュに読み込む Subroutine
sub fasHashReader
{
  my $name = "";
  my $seq = "";
  my $seqLength = "";
  my $stone = 0;
  my @nameArraySR = ( );
  my %fasHashSR = ( );

  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine =~ /^>/){  
      if($stone == 0){$name = $inFileLine;
        $stone = 1;  
      } else {
        $fasHashSR{$name} = $seq;  
        push(@nameArraySR,$name);
        $name = "$inFileLine";
        $seq = "";
      }
    } else {
      $seq .= $inFileLine; 
    }
  }
  $fasHashSR{$name} = $seq;
  $seqLength = length($seq);
  push(@nameArraySR,$name);

  # ハッシュと配列をメインプログラムに返す．
  return (\%fasHashSR,\@nameArraySR,\$seqLength);

}


# Fasta 形式の cDNA ファイルを Nexus 形式に変換するファイル．
# sub fasHashReader も使うので，両方ペーストすること．
sub FasToNex_cDNA
{
    my ($infile) = @_;

    open(INFILE,"$infile");
    my @inFileCont = <INFILE>;
    close INFILE;
    
    my ($ref_fasHashSR,$ref_nameArraySR, $seqLength) = &fasHashReader(\@inFileCont);
    my %efas = %$ref_fasHashSR;
    my @SpListArray = @$ref_nameArraySR;

    
    #### 配列の長さが違うときのために，最長の長さを調べて，短いものはギャップで埋める．
    # この操作は，fasta 形式の配列の長さが違う場合を考慮したもの．
    my $maxSeqLength = "";
    my @seqLength = ( );
    my @seqLengthSorted = ( );

    foreach (values %efas){
      push (@seqLength,length($_));
    }
    @seqLengthSorted = sort { $a <=> $b } @seqLength;
    #print "@seqLengthSorted\n";
    $maxSeqLength = pop(@seqLengthSorted);
    #print "max $maxSeqLength\n";
    
    
    ##### 最も長い名前の文字数を調べて，名前の後に加える空白の数を決める．
    my $maxNameLength = "";
    my @nameLength = ( );
    my @nameLengthSorted = ( );

    foreach (@SpListArray) {
      push(@nameLength,length($_));  
    }
    @nameLengthSorted = sort { $a <=> $b } @nameLength;
    #print "@nameLengthSorted\n";
    $maxNameLength = pop(@nameLengthSorted);
    #print "Max name length $maxNameLength\n";
    
    
    
    #######プリントアウト
    
    my $space = "";
    my $gap = "";
    my $outfile = $infile.".nex";

    open(OUT,">$outfile");
    
print OUT 
"#NEXUS 

BEGIN DATA;
	DIMENSIONS  NTAX=".($#SpListArray+1)." NCHAR= $maxSeqLength;
	FORMAT DATATYPE=PROTEIN  SYMBOLS = \" 1 2 3 4\"  MISSING=? GAP=- ;

MATRIX\n";
    

foreach my $pname (@SpListArray) {
  chomp($pname);
  $space = " " x ($maxNameLength + 5 -length($pname));
  $gap = "-" x ($maxSeqLength - length($efas{$pname}));
  my $otuNameNex = $pname;
  $otuNameNex =~ s/^>//;
  print OUT "$otuNameNex     $space $efas{$pname}".$gap."\n";
}
  
print OUT ";
END;



BEGIN CODONS;
	CODESET  * UNTITLED = Universal: all ;
END;


BEGIN ASSUMPTIONS;
	OPTIONS  DEFTYPE=unord PolyTcount=MINSTEPS ;
END;


BEGIN MacClade;
	Version 4.0  87;
	LastModified -921136733;
	FileSettings editor  '0' '0' '1' '1';
	Singles 100;
	Editor 00011001111111100100010010 '1' '12' Geneva '9' '192' '1' all;
	EditorPosition  '238' '151' '547' '1127';
	TreeWindowPosition  '46' '6' '835' '1390';
	ListWindow Characters closed Geneva '9' '50' '10' '129' '352' 000;
	ListWindow Taxa closed Geneva '9' '50' '10' '129' '490' 100000000;
	ListWindow Trees closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow TypeSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow WtSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow ExSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow TaxSets closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharPartitions closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow CharPartNames closed Geneva '9' '50' '10' '276' '490' ;
	ListWindow WtSets closed Geneva '9' '50' '10' '276' '490' ;
	ChartWindowPosition  '52' '30' '818' '1380';
	StateNamesSymbols closed Geneva '9' '10' '50' '30' '148' '220';
	WindowOrder  Data;
	OtherSymbols &/ 00 ?-;
	Correlation  '0' '0' '1000' '0' '0' 10011010;
	Salmo 00000001;
	EditorFile  '2';
	ExportHTML _ MOSS  '100' 110000;
	PrettyPrint 10;
	EditorToolsPosition  '705' '46' '115' '165';
	TreeWindowProgram 10;
	TreeWindow 0000;
	Continuous  '0' '3' 1;
	Calculations 0000001;
	SummaryMode  '0' '0' 0;
	Charts  Geneva '9' (normal) 0010;
	NexusOptions  '0' '0' '50' 001011001;
	TipLabel  '1';
	TreeFont  Geneva '9' (normal);
	TreeShape  1.0 1.0 1110;
	TraceLabels 0101;
	ChartColors  '0' '0' '65535' '9' '0' 1;
	ChartBiggestSpot 1;
	ChartPercent 10;
	ChartBarWidth  '10' 1;
	ChartVerticalAxis 10101;
	ChartMinMax  '0';
	TraceAllChangesDisplay  '1' 1;
	BarsOnBranchesDisplay  '0' '0' '60000' '10000' '10000' '10000' '10000' '60000' '65000' '65000' '65000' '6' '1' 0000101;
	ContinuousBranchLabels 0;
	AllStatesBranchLabels 1;
	IndexNotation  '2' 1;
	PrintTree  10.00 '2' '2' '2' '2' '2' '2' '2' '2' '2' '2' '2' Geneva '9' (normal) Geneva '10' (normal) Geneva '9' (normal) Geneva '9' (normal) Geneva '9' (bold ) Geneva '9' (normal) Geneva '9' (normal) '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '0' '1' '1' '1' '1' '1' '-26269' '-39' '4' '-40' '0' '1' '2' '1' '8' '0' '0' '0' '2' 1000111000000000000100000111000;
	MatchChar 00 .;
	EntryInterpretation 01;
	ColorOptions 00;
	TreeTools  '0' '5' '4' '0' '10' '4' '0' 00100111111101110;
	EditorTools  '0' '0' '0' '1000' '0' '0' '6' '3' '1' 100010101110011;
	PairAlign  '2' '2' '3' '2' '1' '1' '2' '1' '3' 1010;
	BothTools  '1';
END;


";
    
    print OUT "\n";
    
    close OUT;
    
    # outfile の改行コードを Unix から　Mac する．
    system ("perl -i -pe 's/\n/\r/g' $outfile");
    
    print "infile: $infile\n";
    print "outfile: $outfile\n";
}